//
// Experimental Brother driver table for LPrint, a Label Printer Application
//
// Copyright © 2023-2024 by Michael R Sweet.
//
// Licensed under Apache License v2.0.  See the file "LICENSE" for more
// information.
//

{ "brother_ql-500",	"Brother QL-500",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-500;", NULL },
{ "brother_ql-550",	"Brother QL-550",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-550;", NULL },
{ "brother_ql-560",	"Brother QL-560",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-560;", NULL },
{ "brother_ql-570",	"Brother QL-570",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-570;", NULL },
{ "brother_ql-580n",	"Brother QL-580N",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-580N;", NULL },
{ "brother_ql-600",	"Brother QL-600",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-600;", NULL },
{ "brother_ql-650td",	"Brother QL-650TD",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-650TD;", NULL },
{ "brother_ql-700",	"Brother QL-700",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-700;", NULL },
{ "brother_ql-710w",	"Brother QL-710W",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-710W;", NULL },
{ "brother_ql-720nw",	"Brother QL-720NW",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-720NW;", NULL },
{ "brother_ql-800",	"Brother QL-800",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-800;", NULL },
{ "brother_ql-810w",	"Brother QL-810W",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-810W;", NULL },
{ "brother_ql-820nwb",	"Brother QL-820NWB",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-820NWB;", NULL },
{ "brother_ql-1050",	"Brother QL-1050",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-1050;", NULL },
{ "brother_ql-1060n",	"Brother QL-1060N",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-1060N;", NULL },
{ "brother_ql-1100",	"Brother QL-1100",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-1100;", NULL },
{ "brother_ql-1110nwb",	"Brother QL-1110NWB",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-1110NWB;", NULL },
{ "brother_ql-1115nwb",	"Brother QL-1115NWB",
  "MFG:Brother;CMD:PT-CBP;MDL:QL-1115NWB;", NULL },
//{ "brother_pt-",	"Brother PT-",
//  "MFG:Brother;CMD:???;MDL:PT-;", NULL },
