//
// Experimental CPCL driver table for LPrint, a Label Printer Application
//
// Copyright © 2024 by Michael R Sweet.
//
// Licensed under Apache License v2.0.  See the file "LICENSE" for more
// information.
//

{ "cpcl-203dpi",		"CPCL Label Printer",
  "", NULL },
