//
// DYMO driver table for LPrint, a Label Printer Application
//
// Copyright © 2019-2023 by Michael R Sweet.
//
// Licensed under Apache License v2.0.  See the file "LICENSE" for more
// information.
//

{ "dymo_lm-400",             "DYMO LabelMANAGER 400",
  "MFG:DYMO;MDL:LabelMANAGER 400 ;", NULL },
{ "dymo_lm-450",             "DYMO LabelMANAGER 450",
  "MFG:DYMO;MDL:LabelMANAGER 450 ;", NULL },
{ "dymo_lm-pc",              "DYMO LabelMANAGER PC",
  "MFG:DYMO;MDL:LabelMANAGER PC ;", NULL },
{ "dymo_lm-pc-ii",           "DYMO LabelMANAGER PC II",
  "MFG:DYMO;MDL:LabelMANAGER PC II ;", NULL },
{ "dymo_lm-pnp",             "DYMO LabelMANAGER PNP",
  "MFG:DYMO;MDL:LabelMANAGER PNP ;", NULL },
{ "dymo_lp-350",             "DYMO LabelPOINT 350",
  "MFG:DYMO;MDL:LabelPOINT 350 ;", NULL },
{ "dymo_lw-300",             "DYMO LabelWriter 300",
  "MFG:DYMO;MDL:LabelWriter 300;", NULL },
{ "dymo_lw-310",             "DYMO LabelWriter 310",
  "MFG:DYMO;MDL:LabelWriter 310;", NULL },
{ "dymo_lw-315",             "DYMO LabelWriter 315",
  "MFG:DYMO;MDL:LabelWriter 315;", NULL },
{ "dymo_lw-320",             "DYMO LabelWriter 320",
  "MFG:DYMO;MDL:LabelWriter 320;", NULL },
{ "dymo_lw-330",             "DYMO LabelWriter 330",
  "MFG:DYMO;MDL:LabelWriter 330;", NULL },
{ "dymo_lw-330-turbo",       "DYMO LabelWriter 330 Turbo",
  "MFG:DYMO;MDL:LabelWriter 330 Turbo;", NULL },
{ "dymo_lw-400",             "DYMO LabelWriter 400",
  "MFG:DYMO;MDL:LabelWriter ;", NULL },
{ "dymo_lw-400-turbo",       "DYMO LabelWriter 400 Turbo",
  "MFG:DYMO;MDL:LabelWriter ;", NULL },
{ "dymo_lw-450",             "DYMO LabelWriter 450",
  "MFG:DYMO;MDL:LabelWriter 450;", NULL },
{ "dymo_lw-450-duo-label",   "DYMO LabelWriter 450 DUO Label",
  "MFG:DYMO;MDL:LabelWriter 450 DUO Label;", NULL },
{ "dymo_lw-450-duo-tape",    "DYMO LabelWriter 450 DUO Tape",
  "MFG:DYMO;MDL:LabelWriter 450 DUO Tape;", NULL },
{ "dymo_lw-450-turbo",       "DYMO LabelWriter 450 Turbo",
  "MFG:DYMO;MDL:LabelWriter 450 Turbo;", NULL },
{ "dymo_lw-450-twin-turbo",  "DYMO LabelWriter 450 Twin Turbo",
  "MFG:DYMO;MDL:LabelWriter 450 Twin Turbo;", NULL },
{ "dymo_lw-4xl",             "DYMO LabelWriter 4XL",
  "MFG:DYMO;MDL:LabelWriter 4XL;", NULL },
{ "dymo_lw-duo-label",       "DYMO LabelWriter DUO Label",
  "MFG:DYMO;MDL:LabelWriter DUO Label;", NULL },
{ "dymo_lw-duo-tape",        "DYMO LabelWriter DUO Tape",
  "MFG:DYMO;MDL:LabelWriter DUO Tape;", NULL },
{ "dymo_lw-duo-tape-128",    "DYMO LabelWriter DUO Tape 128",
  "MFG:DYMO;MDL:LabelWriter DUO Tape 128;", NULL },
{ "dymo_lw-se450",           "DYMO LabelWriter SE450",
  "MFG:DYMO;MDL:LabelWriter SE450;", NULL },
//{ "dymo_lw-wireless",        "DYMO LabelWriter Wireless", "", NULL },
//{ "dymo_lw-550",             "DYMO LabelWriter 550", "", NULL },
//{ "dymo_lw-550-turbo",       "DYMO LabelWriter 550 Turbo", "", NULL },
//{ "dymo_lw-5xl",             "DYMO LabelWriter 5XL", "", NULL },
