//
// EPL2 driver table for LPrint, a Label Printer Application
//
// Copyright © 2019-2023 by Michael R Sweet.
//
// Licensed under Apache License v2.0.  See the file "LICENSE" for more
// information.
//

{ "epl2_2inch-203dpi-dt",        "Zebra EPL Page Mode 2-inch/203dpi/Direct-Thermal",
  "", NULL },
{ "epl2_2inch-203dpi-tt",        "Zebra EPL Page Mode 2-inch/203dpi/Thermal-Transfer",
  "", NULL },
{ "epl2_2inch-300dpi-dt",        "Zebra EPL Page Mode 2-inch/300dpi/Direct-Thermal",
  "", NULL },
{ "epl2_2inch-300dpi-tt",        "Zebra EPL Page Mode 2-inch/300dpi/Thermal-Transfer",
  "", NULL },
{ "epl2_4inch-203dpi-dt",        "Zebra EPL Page Mode 4-inch/203dpi/Direct-Thermal",
  "COMMAND SET:EPL;", NULL },
{ "epl2_4inch-203dpi-tt",        "Zebra EPL Page Mode 4-inch/203dpi/Thermal-Transfer",
  "", NULL },
{ "epl2_4inch-300dpi-dt",        "Zebra EPL Page Mode 4-inch/300dpi/Direct-Thermal",
  "", NULL },
{ "epl2_4inch-300dpi-tt",        "Zebra EPL Page Mode 4-inch/300dpi/Thermal-Transfer",
  "", NULL },
