//
// TSPL driver table for LPrint, a Label Printer Application
//
// Copyright © 2023 by Michael R Sweet.
//
// Licensed under Apache License v2.0.  See the file "LICENSE" for more
// information.
//

{ "tspl_203dpi",		"203dpi TSPL Label Printer", NULL, NULL },
{ "tspl_rollo-x1038_203dpi",	"Rollo X1038",
  "MFG:Printer;MDL:Thermal Printer;CMD:XPP,XL;", NULL },
{ "tspl_300dpi",		"300dpi TSPL Label Printer", NULL, NULL },
