//
// Seiko Instruments, Inc. driver table for LPrint, a Label Printer Application
//
// Copyright © 2023 by Michael R Sweet.
//
// Licensed under Apache License v2.0.  See the file "LICENSE" for more
// information.
//

// driver-name			make-and-model	device-id
{ "sii_slp100_203dpi",		"SII SLP100",	"MFG:SII;CMD:SIISLP;MDL:SLP100;", NULL },
{ "sii_slp200_203dpi",		"SII SLP200",	"MFG:SII;CMD:SIISLP;MDL:SLP200;", NULL },
{ "sii_slp240_203dpi",		"SII SLP240",	"MFG:SII;CMD:SIISLP;MDL:SLP240;", NULL },
{ "sii_slp410_203dpi",		"SII SLP410",	"MFG:SII;CMD:SIISLP;MDL:SLP410;", NULL },
{ "sii_slp420_203dpi",		"SII SLP420",	"MFG:SII;CMD:SIISLP;MDL:SLP420;", NULL },
{ "sii_slp430_203dpi",		"SII SLP430",	"MFG:SII;CMD:SIISLP;MDL:SLP430;", NULL },
{ "sii_slp440_300dpi",		"SII SLP440",	"MFG:SII;CMD:SIISLP;MDL:SLP440;", NULL },
{ "sii_slp450_300dpi",		"SII SLP450",	"MFG:SII;CMD:SIISLP;MDL:SLP450;", NULL },
{ "sii_slp620_203dpi",		"SII SLP620",	"MFG:SII;CMD:SIISLP;MDL:SLP620;", NULL },
{ "sii_slp650_300dpi",		"SII SLP650",	"MFG:SII;CMD:SIISLP;MDL:SLP650;", NULL },
{ "sii_slp650se_300dpi",	"SII SLP650SE",	"MFG:SII;CMD:SIISLP;MDL:SLP650SE;", NULL },
