//
// ZPL driver table for LPrint, a Label Printer Application
//
// Copyright © 2019-2023 by Michael R Sweet.
//
// Licensed under Apache License v2.0.  See the file "LICENSE" for more
// information.
//

{ "zpl_2inch-203dpi-dt",	"Zebra ZPL 2-inch/203dpi/Direct-Thermal",
  "", NULL },
{ "zpl_2inch-203dpi-dt_zd611",	"Zebra ZD611d 203dpi",
  "MANUFACTURER:Zebra Technologies ;COMMAND SET:ZPL;MODEL:ZTC ZD611-203dpi ZPL;", NULL },
{ "zpl_2inch-203dpi-tt",	"Zebra ZPL 2-inch/203dpi/Thermal-Transfer",
  "", NULL },
{ "zpl_2inch-300dpi-dt",	"Zebra ZPL 2-inch/300dpi/Direct-Thermal",
  "", NULL },
{ "zpl_2inch-300dpi-dt_zd611",	"Zebra ZD611d 300dpi",
  "MANUFACTURER:Zebra Technologies ;COMMAND SET:ZPL;MODEL:ZTC ZD611-300dpi ZPL;", NULL },
{ "zpl_2inch-300dpi-tt",	"Zebra ZPL 2-inch/300dpi/Thermal-Transfer",
  "", NULL },
{ "zpl_2inch-600dpi-tt",	"Zebra ZPL 2-inch/600dpi/Thermal-Transfer",
  "", NULL },
{ "zpl_4inch-203dpi-dt",	"Zebra ZPL 4-inch/203dpi/Direct-Thermal",
  "", NULL },
{ "zpl_4inch-203dpi-dt_gx420d",	"Zebra GX420d",
  "MANUFACTURER:Zebra Technologies;COMMAND SET:ZPL;MODEL:ZTC GX420d;", NULL },
{ "zpl_4inch-203dpi-dt_gx430d",	"Zebra GX430d",
  "MANUFACTURER:Zebra Technologies;COMMAND SET:ZPL;MODEL:ZTC GX430d;", NULL },
{ "zpl_4inch-203dpi-dt_zd621",	"Zebra ZD621d 203dpi",
  "MANUFACTURER:Zebra Technologies ;COMMAND SET:ZPL;MODEL:ZTC ZD621-203dpi ZPL;", NULL },
{ "zpl_4inch-203dpi-tt",	"Zebra ZPL 4-inch/203dpi/Thermal-Transfer",
  "", NULL },
{ "zpl_4inch-203dpi-tt_gx420t",	"Zebra GX420t",
  "MANUFACTURER:Zebra Technologies;COMMAND SET:ZPL;MODEL:ZTC GX420t;", NULL },
{ "zpl_4inch-203dpi-tt_gx430t",	"Zebra GX430t",
  "MANUFACTURER:Zebra Technologies;COMMAND SET:ZPL;MODEL:ZTC GX430t;", NULL },
{ "zpl_4inch-300dpi-dt",	"Zebra ZPL 4-inch/300dpi/Direct-Thermal",
  "", NULL },
{ "zpl_4inch-300dpi-dt_zd621",	"Zebra ZD621d 300dpi",
  "MANUFACTURER:Zebra Technologies ;COMMAND SET:ZPL;MODEL:ZTC ZD621-300dpi ZPL;", NULL },
{ "zpl_4inch-300dpi-tt",	"Zebra ZPL 4-inch/300dpi/Thermal-Transfer",
  "", NULL },
{ "zpl_4inch-600dpi-tt",	"Zebra ZPL 4-inch/600dpi/Thermal-Transfer",
  "", NULL },
